@@ -15,10 +15,14 @@ Estimation of power systems weight
 #  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 import numpy as np
-from openmdao.core.explicitcomponent import ExplicitComponent
+import openmdao.api as om
+
+from fastoad.module_management.service_registry import RegisterSubmodel
+from .constants import SERVICE_POWER_SYSTEMS_MASS
 
 
-class PowerSystemsWeight(ExplicitComponent):
+@RegisterSubmodel(SERVICE_POWER_SYSTEMS_MASS, "fastoad.submodel.weight.mass.systems.power.legacy")
+class PowerSystemsWeight(om.ExplicitComponent):
     """
     Weight estimation for power systems (generation and distribution)
 
